#include <termios.h>
#include <stdlib.h>
#include <unistd.h> 
#include <sys/types.h> 
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/time.h>
#include "usbtmc.h" 
#include "gmt_function.h"

static int usbtmc_fd = -1;
struct usbtmc_instrument inst;
struct usbtmc_attribute attr;

int usbtmcInit(void)
{
	usbtmc_fd = open("/dev/usbtmc0", O_RDWR);
	if (usbtmc_fd == -1)
	{	  
	  Display_Message("usbtmcInit() Open Failure... 1234\n");	
	  Start_Button(2);		
//	  exit(1);
	}
	printf("usbtmc Init success...\n");
	return 0;	
}
//*************************************************************************************
void usbtmcDone(void)
{
	if (usbtmc_fd >= 0)
	{
		close(usbtmc_fd);
		usbtmc_fd = -1;
	}
}
//*************************************************************************************
void usbtmcIoctl(int usbtmc_fd, unsigned int cmd)
{
	if (cmd == USBTMC_ATTRIB_READ_MODE)
	{
		attr.attribute=USBTMC_ATTRIB_READ_MODE;
		attr.value=USBTMC_ATTRIB_VAL_READ;
		ioctl(usbtmc_fd, cmd, &attr);
	}
	else if (cmd == USBTMC_ATTRIB_NUM_INSTRUMENTS)
	{
		attr.attribute=USBTMC_ATTRIB_NUM_INSTRUMENTS;
		ioctl(usbtmc_fd, USBTMC_IOCTL_GET_ATTRIBUTE, &attr);
		printf("Number of instruments active: %d\n",attr.value);
	}
}
//*************************************************************************************
int usbtmcWrite(char* buf, unsigned int count)
{
	return write(usbtmc_fd, buf, count);
}
//*************************************************************************************
int usbtmcRead(char* buf, unsigned int count)
{
	return read(usbtmc_fd, buf, count);
}

/*
 * return:
 *   0 --- error
 *   1 --- OK
 */
unsigned char read_rgb(float *x,float *y,float *z)
{
	unsigned char buf[17]={0};
	int i;

	buf[0]=0x06;
	buf[1]=0x00;
	buf[2]=0x00;
	buf[3]=0x00;
	buf[4]=0x00;
	
	usbtmcWrite(buf,5);
	usbtmcRead(buf,17);
//	for(i=0;i<17;i++)
//		printf("%x ",buf[i]);
	for(i=0;i<4;i++)
	{
		((unsigned char*)x)[i]=buf[3+i];
		((unsigned char*)y)[i]=buf[7+i];
		((unsigned char*)z)[i]=buf[11+i];
	}
//	printf("---\nx,y,z=%f,%f,%f\n",*x,*y,*z);
	return 1;
}
/*
unsigned char read_rgb(float *x,float *y,float *z)
{
	char buf[17]={0};
	int i;

	buf[0]=0x05;
	buf[1]=0x00;
	buf[2]=0x00;
	buf[3]=0x00;
	buf[4]=0x00;
	
	usbtmcWrite(buf,5);
	usbtmcRead(buf,17);
	for(i=0;i<17;i++)
		printf("%x ",buf[i]);
	for(i=0;i<4;i++)
	{
		((char*)x)[i]=buf[3+i];
		((char*)y)[i]=buf[7+i];
		((char*)z)[i]=buf[11+i];
	}
	printf("---\nx,y,z=%f,%f,%f\n",*x,*y,*z);
	return 1;
}
*/
/*
 * return:
 *   0 -- error
 *  !0 -- flick value
 */
float read_flick(void)         //32-bit
{
	float flick=0.0;
	unsigned char buf[17]={0};
	int i;
	
	buf[0]=0x09;
	buf[1]=0x00;
	buf[2]=0x00;
	buf[3]=0x00;
	buf[4]=0x00;
	usbtmcWrite(buf,5);
	usbtmcRead(buf,9);

//	for(i=0;i<9;i++)
//		printf("%x ",buf[i]);
	for(i=0;i<4;i++)
	{
		((unsigned char*)&flick)[i] = buf[3+i];
	}
//	printf("\nflick=%f\n",flick);
	return flick;
 }

//
void read_max_min(float *max,float *min)         //32-bit
{
	float *p;
	unsigned char buf[17]={0x12,0x00,0x00,0x00,0x00};
	
	usbtmcWrite(buf,5);
	usbtmcRead(buf,13);//8+5
	p=max;
	p[0]=buf[3];
	p[1]=buf[4];
	p[2]=buf[5];
	p[3]=buf[6];
	p=min;
	p[0]=buf[7];
	p[1]=buf[8];
	p[2]=buf[9];
	p[3]=buf[10];
}

//
float read_flick_value(void)         //32-bit
{
	float max,min;
	unsigned char buf[17]={0},*p;
	
	buf[0]=0x12;
	usbtmcWrite(buf,5);
	usbtmcRead(buf,13);//8+5
	p=&max;
	p[0]=buf[3];
	p[1]=buf[4];
	p[2]=buf[5];
	p[3]=buf[6];
	p=&min;
	p[0]=buf[7];
	p[1]=buf[8];
	p[2]=buf[9];
	p[3]=buf[10];
	
	return max-min;
}

void read_all_flick_value(void)         //32-bit
{
	//float flick[256]={0};
	float *p;
	float dat;
	unsigned char buf[17]={0};
	int i,j;
	
	for(j=0;j<255;j++)
	{
		buf[0]=0x11;
		buf[1]=0x00;
		buf[2]=0x00;
		buf[3]=0x00;
		buf[4]=0x00;
		usbtmcWrite(buf,5);
		usbtmcRead(buf,17);//12+5
		//p=&flick[j];
		p=&dat;
		for(i=0;i<4;i++)
		{
			((unsigned char*)p)[i] = buf[3+i];
		}
		Test_Float("flicker = %.4f",dat,-5000,5000);
	}
	//other  operation
	//
	//
 }
char DarkCalib(void)
{
	unsigned char buf[17]={0};
	float x,y,z;
	buf[0]=0x13;
	buf[1]=0x00;
	buf[2]=0x00;
	buf[3]=0x00;
	buf[4]=0x00;
	read_rgb(&x,&y,&z);
	usbtmcWrite(buf,5);
	Wait(30);
	usbtmcRead(buf,3);
	return 0;
}
